from django.core.exceptions import ObjectDoesNotExist
from edc_action_item.model_wrappers import (
    ActionItemModelWrapper as BaseActionItemModelWrapper,
)
from edc_constants.constants import NEW

from ...utils import get_ae_model
from ..death_report_model_wrapper import DeathReportModelWrapper
from ..death_report_tmg_model_wrapper import DeathReportTmgModelWrapper


class TmgActionItemModelWrapper(BaseActionItemModelWrapper):
    def __init__(self, model_obj=None, **kwargs):
        self._death_report = None
        self._tmg_death_report = None
        super().__init__(model_obj=model_obj, **kwargs)

    @property
    def death_report_tmg_verbose_name(self):
        return self.death_report_tmg_model_cls._meta.verbose_name

    @property
    def death_report_model_cls(self):
        return get_ae_model("deathreport")

    @property
    def death_report_tmg_model_cls(self):
        return get_ae_model("deathreporttmg")

    @property
    def death_report(self):
        if not self._death_report:
            try:
                self._death_report = DeathReportModelWrapper(
                    model_obj=self.death_report_model_cls.objects.get(
                        subject_identifier=self.subject_identifier
                    )
                )
            except ObjectDoesNotExist:
                self._death_report = None
        return self._death_report

    def get_wrapped_tmg_death_report(self, tmg_death_report=None):
        if tmg_death_report:
            wrapped = DeathReportTmgModelWrapper(
                model_obj=self.death_report_tmg_model_cls.objects.get(
                    action_identifier=self.object.action_identifier,
                    death_report=self.death_report.object,
                )
            )
        else:
            wrapped = DeathReportTmgModelWrapper(
                model_obj=self.death_report_tmg_model_cls(
                    death_report=self.death_report.object,
                    subject_identifier=self.object.subject_identifier,
                    action_identifier=self.object.action_identifier,
                    parent_action_item=self.object.parent_action_item,
                    related_action_item=self.object.related_action_item,
                )
            )
        return wrapped

    @property
    def tmg_death_report(self):
        """Returns a wrapped new or existing Death Report TMG
        model instance.

        There can only be one instance per action item.
        """
        if not self._tmg_death_report:
            if self.death_report:
                if self.object.status == NEW:
                    self._tmg_death_report = self.get_wrapped_tmg_death_report()
                else:
                    model_obj = self.death_report_tmg_model_cls.objects.get(
                        action_identifier=self.object.action_identifier,
                        death_report=self.death_report.object,
                    )
                    self._tmg_death_report = self.get_wrapped_tmg_death_report(model_obj)
        return self._tmg_death_report



from .closed_tmg_action_item_model_wrapper import ClosedTmgActionItemModelWrapper
from .new_tmg_action_item_model_wrapper import NewTmgActionItemModelWrapper
from .open_tmg_action_item_model_wrapper import OpenTmgActionItemModelWrapper
from .tmg_action_item_model_wrapper import TmgActionItemModelWrapper



from edc_model_wrapper import ModelWrapper

from ..utils import get_adverse_event_app_label


class AeInitialModelWrapper(ModelWrapper):
    model = f"{get_adverse_event_app_label()}.aeinitial"
    next_url_name = "ae_listboard_url"
    next_url_attrs = ["subject_identifier"]

    @property
    def pk(self):
        return str(self.object.pk)

    @property
    def subject_identifier(self):
        return self.object.subject_identifier



from __future__ import annotations

from typing import TYPE_CHECKING, Type

from django import forms
from django.apps import apps as django_apps
from django.conf import settings
from edc_utils import convert_php_dateformat

if TYPE_CHECKING:
    from edc_adverse_event.model_mixins import (
        AeFollowupModelMixin,
        AeInitialModelMixin,
        DeathReportModelMixin,
    )


def validate_ae_initial_outcome_date(form_obj):
    ae_initial = form_obj.cleaned_data.get("ae_initial")
    if not ae_initial and form_obj.instance:
        ae_initial = form_obj.instance.ae_initial
    outcome_date = form_obj.cleaned_data.get("outcome_date")
    if ae_initial and outcome_date:
        if outcome_date < ae_initial.ae_start_date:
            formatted_dte = ae_initial.ae_start_date.strftime(
                convert_php_dateformat(settings.SHORT_DATE_FORMAT)
            )
            raise forms.ValidationError(
                {"outcome_date": f"May not be before the AE start date {formatted_dte}."}
            )


def get_adverse_event_app_label() -> str:
    app_label = getattr(settings, "ADVERSE_EVENT_APP_LABEL", None)
    if not app_label:
        app_label = getattr(settings, "EDC_ADVERSE_EVENT_APP_LABEL", None)
    if not app_label:
        raise ValueError(
            "Attribute not set. See `get_adverse_event_app_label()` or "
            "`settings.EDC_ADVERSE_EVENT_APP_LABEL`."
        )
    return app_label


def get_hospitalization_model_app_label() -> str:
    return getattr(
        settings, "EDC_ADVERSE_EVENT_HOSPITALIZATION_APP_LABEL", get_adverse_event_app_label()
    )


def get_ae_model(
    model_name,
) -> Type[DeathReportModelMixin] | Type[AeInitialModelMixin] | Type[AeFollowupModelMixin]:
    return django_apps.get_model(f"{get_adverse_event_app_label()}.{model_name}")


def get_ae_model_name(model_name) -> str:
    return f"{get_adverse_event_app_label()}.{model_name}"



from edc_model_wrapper import ModelWrapper

from ..utils import get_adverse_event_app_label


class AeTmgModelWrapper(ModelWrapper):
    next_url_name = "tmg_ae_listboard_url"
    model = f"{get_adverse_event_app_label()}.aetmg"
    next_url_attrs = ["subject_identifier"]

    @property
    def pk(self):
        return str(self.object.pk)

    @property
    def subject_identifier(self):
        return self.object.subject_identifier



from edc_action_item import ActionWithNotification
from edc_constants.constants import HIGH_PRIORITY

from ..constants import HOSPITALIZATION_ACTION


class HospitalizationAction(ActionWithNotification):
    reference_model: str = None  # e.g. "intecomm_prn.hospitalization"
    admin_site_name: str = None  # e.g. "intecomm_prn_admin"

    name = HOSPITALIZATION_ACTION
    display_name = "Submit Hospitalization Report"
    notification_display_name = "Hospitalization"
    parent_action_names = []
    show_link_to_changelist = True
    show_link_to_add = True
    priority = HIGH_PRIORITY
